/* ******************************************************************

   class    : ClassPane
   date     : Sep 29 1998
   compiler : javac (1.1.6)

******************************************************************* */

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.tree.*;

import SID.Meta.*;

import Utils.ResourceUtil;

/**
 * This class is the visual representation of the class browser.
 *
 * @author puehringer edgar
 */
public class ClassPane extends JScrollPane implements Runnable {

  // instance variable declarations
  final JTree tree;
  DefaultTreeModel classes;
  Hashtable listeners = new Hashtable(1);
  TreePath firstClass;

  // constructors
  public ClassPane(ResourceBundle resources, DefaultTreeModel model) {
    super();

    ClassTreeRenderer rend = new ClassTreeRenderer(resources);
    JViewport port;
    MouseListener ml;

    classes = model;
    rend.setOpaque(true);
    tree = new JTree(classes);
    port = getViewport();
    port.add(tree);
    tree.setCellRenderer(rend);
    ml = new MouseAdapter() {

      public void mouseClicked(MouseEvent e) {
        int selRow       = tree.getRowForLocation(e.getX(), e.getY());
        TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());

        if ((selRow != -1) && (e.getClickCount() == 2)) {
          onDoubleClick(selRow,selPath);
        } // end if
      } // end mouseClicked

    };
    tree.addMouseListener(ml);
    tree.setEditable(false);
  } // end constructor ClassPane

  // methods
  /** Invoked on a double click on a tree node. */
  protected void onDoubleClick(int selRow, TreePath selPath) {
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)selPath.getLastPathComponent();
    Object obj                  = node.getUserObject();

    if ((obj instanceof MethodInfo) ||
        (obj instanceof AttributeInfo)) {
      NamedItem ni     = (NamedItem)obj;
      int line         = ni.position - 1;
      String cmd;
      Enumeration enum = listeners.keys();

      while (!(obj instanceof ClassInfo)) {
        node = (DefaultMutableTreeNode)node.getParent();
        obj  = node.getUserObject();
      } // end while
      ni = (NamedItem)obj;
      cmd = line + " " + ni.fileName;
      while (enum.hasMoreElements()) {
        ActionListener al = (ActionListener)enum.nextElement();
        ActionEvent ae;

        ae = new ActionEvent(this,ActionEvent.ACTION_PERFORMED,cmd);
        al.actionPerformed(ae);
      } // end while
    } // end if
  } // end onDoubleClick

  /** Expand the subtrees of all classes that are implemented in file. */
  public void expandClassesOfFile(File file) {
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)classes.getRoot();

    firstClass = null;
    if (file == null) {
      getToolkit().beep();
      return;
    } // end if
    firstClass = null;
    internalExpand(file,node);
    if (firstClass != null) {
      SwingUtilities.invokeLater(new Thread(this));
    } // end if
  } // end expandClassesOfFile

  private boolean internalExpand(File file, DefaultMutableTreeNode node) {
    Object obj       = node.getUserObject();
    boolean expanded = false;

    for (int i=0; i<node.getChildCount(); i++) {
      if (internalExpand(file,(DefaultMutableTreeNode)node.getChildAt(i))) {
        expanded = true;
      } // end if
    } // end for
    if ((obj instanceof String) || (obj == null)) {
      TreePath path = getTreePath(node);

      if (tree.isExpanded(path) && !expanded) tree.collapsePath(path);
    } else if (obj instanceof ClassInfo) {
      TreePath path = getTreePath(node);
      ClassInfo ci  = (ClassInfo)obj;
      String fPath  = (new File(ci.fileName)).getAbsolutePath();

      if ((firstClass == null) && fPath.equals(file.getAbsolutePath())) {
        firstClass = path;
      } // end if
      if (fPath.equals(file.getAbsolutePath()) || expanded) {
        if (tree.isCollapsed(path)) tree.expandPath(path);
        expanded = true;
      }
      else {
        if (tree.isExpanded(path)) tree.collapsePath(path);
      }
    } // end if
    return expanded;
  } // end internalExpand

  private final TreePath getTreePath(DefaultMutableTreeNode node) {
    return new TreePath(classes.getPathToRoot(node));
  } // end getTreePath
  /**
   * Adds the specified action listener to receive action events.
   *
   * @param l The action listener.
   * @see removeActionListener
   */
  public synchronized void addActionListener(ActionListener l) {
    listeners.put(l,new Object());
  } // end addActionListener

  /**
   * Removes the specified action listener so that it no longer
   * receives action events.
   *
   * @param l The action listener.
   * @see addActionListener
   */
  public synchronized void removeActionListener(ActionListener l) {
    listeners.remove(l);
  } // end removeActionListener

  //---- Runnable --------------------------------------------------
  public void run() {
    DefaultMutableTreeNode node;

    node = (DefaultMutableTreeNode)firstClass.getLastPathComponent();
    try {
      node = (DefaultMutableTreeNode)node.getLastChild();
      tree.scrollPathToVisible(getTreePath(node));
    } catch (NoSuchElementException e) {}
    tree.scrollPathToVisible(firstClass);
    firstClass = null;
  } // end run

} // end ClassPane

/** The TreeCellRenderer for the class browser tree. */
class ClassTreeRenderer extends JLabel implements TreeCellRenderer {

  // class variable declarations
  // Keys to the class browser mini pictures
  private final static String ICON_PACKAGE = "browser.packageIcon";
  private final static String ICON_VCLASS  = "browser.valClassIcon";
  private final static String ICON_ICLASS  = "browser.invClassIcon";
  private final static String ICON_APUB    = "browser.attrPubIcon";
  private final static String ICON_APROT   = "browser.attrProtIcon";
  private final static String ICON_APRIV   = "browser.attrPrivIcon";
  private final static String ICON_MPUB    = "browser.methPubIcon";
  private final static String ICON_MPROT   = "browser.methProtIcon";
  private final static String ICON_MPRIV   = "browser.methPrivIcon";

  /** True if mini pictures allready loaded. */
  private static boolean loadDone = false;
  private static Icon packIcon;
  private static Icon valClass,invClass;
  private static Icon attrPub,attrProt,attrPriv;
  private static Icon methPub,methProt,methPriv;
  // instance variable declarations
  private Border border;

  ClassTreeRenderer(ResourceBundle resources) {
    super();
    if (!loadDone) {
      packIcon = loadImage(resources,ICON_PACKAGE);
      valClass = loadImage(resources,ICON_VCLASS);
      invClass = loadImage(resources,ICON_ICLASS);
      attrPub  = loadImage(resources,ICON_APUB);
      attrProt = loadImage(resources,ICON_APROT);
      attrPriv = loadImage(resources,ICON_APRIV);
      methPub  = loadImage(resources,ICON_MPUB);
      methProt = loadImage(resources,ICON_MPROT);
      methPriv = loadImage(resources,ICON_MPRIV);
      loadDone = true;
    } // end if
    border = BorderFactory.createEmptyBorder(1,1,1,1);
  } // end constructor ClassTreeRenderer

  // methods
  private final ImageIcon loadImage(ResourceBundle resources, String key) {
    String str = ResourceUtil.getResourceString(resources,key);

    if (str == null) return null;
    return new ImageIcon(getClass().getResource(str));
  } // end loadImage

  public Component getTreeCellRendererComponent(JTree tree, Object value,
                     boolean sel, boolean expanded, boolean leaf, int row,
                     boolean hasFocus) {
    Object obj = ((DefaultMutableTreeNode)value).getUserObject();

    if (obj instanceof DocumentedItem){
      DocumentedItem d = (DocumentedItem)obj;

      setBackground(sel ?
        SystemColor.textHighlight :
        SystemColor.window);
      setForeground(sel ?
        SystemColor.textHighlightText :
        SystemColor.windowText);
      setText(d.toString());
      if (d instanceof ClassInfo) {
        if (((ClassInfo)d).valid) setIcon(valClass);
          else setIcon(invClass);
      } else if (d instanceof MethodInfo) {
        if (d.isPrivate()) {
          setIcon(methPriv);
        } else if (d.isPublic()) {
          setIcon(methPub);
        } else {
          setIcon(methProt);
        } // end if
      } else if (d instanceof AttributeInfo) {
        if (d.isPrivate()) {
          setIcon(attrPriv);
        } else if (d.isPublic()) {
          setIcon(attrPub);
        } else {
          setIcon(attrProt);
        } // end if
      } // end if
      setBorder(border);
      return this;
    } // end if
    return new JLabel(obj.toString(),packIcon,JLabel.RIGHT);
  } // end getTreeCellRendererComponent

} // end ClassTreeRenderer

/* ******************************************************************
   end of file
******************************************************************* */
