/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : AutoSaveDlg
   date       : 18.05.98
   description: a dialog that allows to specify auto save options.
   modified   : 6.10.98 - Now in package Dialogs, extends BaseDialog.

*********************************************************************/

package Dialogs;

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import CustCtrl.BaseDialog;
import CustCtrl.IntTextField;
import Utils.ResourceUtil;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class AutoSaveDlg extends BaseDialog
  implements ActionListener, ItemListener, TextListener {

  // class variable declarations
  private final static String  TITLE       = "title";
  private final static String  BUTTONS     = "buttons";
  private final static String  LABEL_CHECK = "enabledLabel";
  private final static String  LABEL_EDIT1 = "saveEveryLabel";
  private final static String  LABEL_EDIT2 = "minutesLabel";
  private final static int     MINUTES     = 5;
  private final static int     MIN_MINUTES = 1;
  private final static int     MAX_MINUTES = 99;

  // instance variable declarations
  private Checkbox enblCheckbox;
  private Label label1;
  private Button downButton;
  private IntTextField textField;
  private Button upButton;
  private Label label2;

  // constructors
  /**
   * Create a modal auto save dialog box.
   *
   * @param parent The parent frame.
   * @param minutes The time between two auto save actions.
   * @param enabled True if auto save is enabled.
   * @param resources A resource bundle to look for widget labels.
   * @param resourcePrefix A string that acts as prefix for resource keys.
   */
  public AutoSaveDlg(Frame parent, int minutes, boolean enabled,
                     ResourceBundle resources, String resourcePrefix) {
    super(parent,true);

    Panel panel = getContentPanel();
    String str;

    setTitle(ResourceUtil.getResStringOrKey(resources,
               resourcePrefix + TITLE));
    createButtons(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + BUTTONS));

    GridBagLayout gridbag          = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    panel.setLayout(gridbag);

    constraints.insets     = new Insets(10,6,10,6);
    constraints.fill       = GridBagConstraints.NONE;
    constraints.anchor     = GridBagConstraints.WEST;
    constraints.weightx    = 1.0;
    constraints.weighty    = 1.0;
    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    constraints.gridheight = GridBagConstraints.RELATIVE;

    str          = ResourceUtil.getResStringOrKey(resources,
                     resourcePrefix + LABEL_CHECK);
    enblCheckbox = new Checkbox(str,enabled);
    enblCheckbox.addItemListener(this);
    gridbag.setConstraints(enblCheckbox,constraints);
    panel.add(enblCheckbox);

    constraints.insets     = new Insets(0,6,10,6);
    constraints.gridwidth  = 1;
    constraints.gridheight = GridBagConstraints.REMAINDER;
    label1 = new Label(ResourceUtil.getResStringOrKey(resources,
               resourcePrefix + LABEL_EDIT1));
    gridbag.setConstraints(label1,constraints);
    panel.add(label1);

    constraints.insets = new Insets(0,0,10,0);
    downButton = new Button("<");
    downButton.addActionListener(this);
    gridbag.setConstraints(downButton,constraints);
    panel.add(downButton);

    textField = new IntTextField(new Integer(minutes),2);
    textField.addTextListener(this);
    gridbag.setConstraints(textField,constraints);
    panel.add(textField);

    constraints.insets    = new Insets(0,0,10,6);
    constraints.gridwidth = GridBagConstraints.RELATIVE;
    upButton = new Button(">");
    upButton.addActionListener(this);
    gridbag.setConstraints(upButton,constraints);
    panel.add(upButton);

    constraints.gridwidth = GridBagConstraints.REMAINDER;
    label2 = new Label(ResourceUtil.getResStringOrKey(resources,
               resourcePrefix + LABEL_EDIT2));
    gridbag.setConstraints(label2,constraints);
    panel.add(label2);
    // actualize the widget states
    itemStateChanged(null);
    pack();
    center(parent);
    setResizable(false);
  } // end constructor AutoSaveDlg

  // methods
  /**
   * Returns the currently selected period of time between two auto save
   * actions.
   */
  public int getMinutes() {
    Integer i = textField.getInteger();

    if (i == null) return MINUTES;
      else return i.intValue();
  } // end getMinutes

  /** Returns true if auto save is enabled. */
  public boolean getEnabled() {
    return enblCheckbox.getState();
  } // end getEnabled

  public void actionPerformed(ActionEvent e) {
    Integer i   = textField.getInteger();
    int minutes = i == null ? MINUTES : i.intValue();

    if (e.getSource().equals(downButton)) {
      if (minutes > MIN_MINUTES) minutes--;
        else getToolkit().beep();
      textField.setInteger(new Integer(minutes));
    }
    else { // if source equals up button
      if (minutes < MAX_MINUTES) minutes++;
        else getToolkit().beep();
      textField.setInteger(new Integer(minutes));
    } // end if
  } // end actionPerformed

  public void itemStateChanged(ItemEvent e) {
    boolean enbl = enblCheckbox.getState();

    textValueChanged(null);
    label1.setEnabled(enbl);
    downButton.setEnabled(enbl);
    textField.setEnabled(enbl);
    upButton.setEnabled(enbl);
    label2.setEnabled(enbl);
  } // end itemStateChanged

  // --- TextListener -----------------------------------------------
  /** Invoked when the value of the text has changed. */
  public void textValueChanged(TextEvent e) {
    Integer i   = textField.getInteger();
    int minutes = i == null ? MINUTES : i.intValue();

    downButton.setEnabled(minutes > MIN_MINUTES);
    upButton.setEnabled(minutes < MAX_MINUTES);
    setButtonEnabled(0,!enblCheckbox.getState() ||
                       ((i != null) &&
                        (minutes >= MIN_MINUTES) &&
                        (minutes <= MAX_MINUTES)));
  } // end textValueChanged

} // end AutoSaveDlg

/* ******************************************************************
   end of file
*********************************************************************/
