/* ******************************************************************

   class    : FileFormatDlg
   date     : 10.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Dialogs;

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.TextListener;
import java.awt.event.TextEvent;
import java.util.ResourceBundle;

import CustCtrl.BaseDialog;
import CustCtrl.IntTextField;
import Utils.ResourceUtil;

/**
 * A dialog that displays application informations.
 *
 * @author puehringer edgar
 */
public class FileFormatDlg extends BaseDialog implements TextListener {

  // class variable declarations
  private final static String TITLE            = "title";
  private final static String BUTTONS          = "buttons";
  private final static String TABLABEL         = "tabSizeLabel";
  private final static String CBLABEL          = "useTabsLabel";
  private final static String FORMATLABEL      = "fileFmtLabel";
  private final static String FORMATOPTIONS    = "fileFmtChoiceOpt";
  private final static String FORMATDLG_PREFIX = "fileFormatDlg.";

  // instance variable declarations
  private Frame parent;
  private ResourceBundle resources;
  private IntTextField text;
  private Checkbox checkbox;
  private Choice choice;
  private boolean okDisabled;

  // constructors
  /** Create a new FileFormatDlg object. */
  public FileFormatDlg(Frame parent, ResourceBundle resources,
                    String resourcePrefix, int tabSize, boolean useTabs,
                    int format) {
    super(parent,true);
    this.parent    = parent;
    this.resources = resources;

    Panel panel = getContentPanel();
    Label label;
    String[] options;

    setTitle(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + TITLE));
    createButtons(ResourceUtil.getResStringOrKey(resources,
                    resourcePrefix + BUTTONS));

    GridBagLayout gridbag          = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    panel.setLayout(gridbag);
    constraints.insets     = new Insets(10,6,0,0);
    constraints.weightx    = 1.0;
    constraints.weighty    = 1.0;
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    constraints.gridheight = 1;
    label = new Label(ResourceUtil.getResStringOrKey(resources,
                      resourcePrefix + TABLABEL));
    gridbag.setConstraints(label,constraints);
    panel.add(label);
    constraints.insets     = new Insets(10,6,0,6);
    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    text = new IntTextField(new Integer(tabSize),3);
    text.addTextListener(this);
    gridbag.setConstraints(text,constraints);
    panel.add(text);
    constraints.insets     = new Insets(10,6,0,6);
    constraints.gridheight = GridBagConstraints.RELATIVE;
    checkbox = new Checkbox(ResourceUtil.getResStringOrKey(resources,
                      resourcePrefix + CBLABEL),useTabs);
    gridbag.setConstraints(checkbox,constraints);
    panel.add(checkbox);
    constraints.insets     = new Insets(10,6,10,0);
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    constraints.gridheight = GridBagConstraints.REMAINDER;
    label = new Label(ResourceUtil.getResStringOrKey(resources,
                      resourcePrefix + FORMATLABEL));
    gridbag.setConstraints(label,constraints);
    panel.add(label);
    constraints.insets     = new Insets(10,6,10,6);
    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    choice = new Choice();
    options = ResourceUtil.splitToWords(ResourceUtil.getResStringOrKey(resources,
                                        resourcePrefix + FORMATOPTIONS));
    for (int i=0; i<options.length; i++) choice.add(options[i]);
    if (--format < choice.getItemCount()) choice.select(format);
    gridbag.setConstraints(choice,constraints);
    panel.add(choice);
    pack();
    center(parent);
    setResizable(false);
  } // end constructor FileFormatDlg

  // class methods
    /** Only for debuging purposes. */
  public static void main(String[] args) throws java.util.MissingResourceException {
    Frame frame = new Frame();
    FileFormatDlg dlg;
    ResourceBundle res;

    if (args.length < 1) {
      System.out.println("Resource file must be specified.");
      System.exit(1);
    } // end if
    res = ResourceBundle.getBundle(args[0],java.util.Locale.getDefault());
    dlg = new FileFormatDlg(frame,res,"fileFormatDlg.",8,true,1);
    dlg.show();
    System.out.println("isCanceled returned " + dlg.isCanceled());
    System.exit(0);
  } // end main

  // methods
  /** Return the tabulator size. */
  public int getTabSize() {
    Integer i = text.getInteger();

    if (i == null) return -1;
    return i.intValue();
  } // end getTabSize

  /** Return if tab characters should be used to store files. */
  public boolean getUseTabs() {
    return checkbox.getState();
  } // end getUseTabs

  /** Return one of the Streams.TabWriter file format constants. */
  public int getFileFormat() {
    return choice.getSelectedIndex() + 1;
  } // end getFileFormat

  public void textValueChanged(TextEvent e) {
    Integer i     = text.getInteger();
    boolean valid = (i != null) && (i.intValue() > 1) && (i.intValue() <= 40);

    if (valid != !okDisabled) {
      okDisabled = !valid;
      setButtonEnabled(0,valid);
    } // end if
  } // end textValueChanged

} // end FileFormatDlg

/* ******************************************************************
   end of file
*********************************************************************/
