/* ******************************************************************

   class    : JMenuTestFrame
   date     : Sep 25 1998
   compiler : javac (1.1.6)

******************************************************************* */

package MenuPreview;

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.io.IOException;
import java.util.*;

import com.sun.java.swing.event.*;
import com.sun.java.swing.*;

/**
 * A class that displays a JFrame with a menu generated from an
 * input stream.
 *
 * @author puehringer edgar
 */
public class JMenuTestFrame extends JFrame
  implements ActionListener {

  // class variable declarations

  // instance variable declarations
  private JTextArea editor;

  // constructors
  /** Create a new JMenuTestFrame object. */
  public JMenuTestFrame(String title, Image iconImg, InputStream is,
                        String menuKey, String toolBarKey, boolean useImages,
                        String imagePath) throws IOException {
    super(title);

    Container cp = getContentPane();
    JPanel panel = new JPanel(new BorderLayout());
    Properties prop;
    JScrollPane scroller;
    JMenuLoader menuLoader;
    JMenuBar menuBar;
    JToolBarLoader toolBarLoader;
    JToolBar toolBar;
    boolean done = false;

    try {
      if (iconImg != null) setIconImage(iconImg);
      setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      prop = new Properties();
      prop.load(is);
      cp.setLayout(new BorderLayout());
      editor = new JTextArea();
      editor.setEditable(false);
      scroller = new JScrollPane();
      scroller.getViewport().add(editor);
      panel.add("Center",scroller);
      if ((toolBarKey != null) && !toolBarKey.equals("")) {
        toolBarLoader = new JToolBarLoader(prop,imagePath) {

          protected void reportError(String text) {
            writeToEditor("JToolBarLoader: " + text + ".");
          } // end reportError

        };
        toolBarLoader.setDefaultActionListener(this);
        toolBar = toolBarLoader.createToolBar(toolBarKey);
        if (toolBar != null) panel.add("North",toolBar);
      } // end if
      cp.add("Center",panel);
      if ((menuKey != null) && !menuKey.equals("")) {
        if (!useImages) imagePath = null;
        menuLoader = new JMenuLoader(prop,imagePath) {

          protected void reportError(String text) {
            writeToEditor("JMenuLoader: " + text + ".");
          } // end reportError

        };
        menuLoader.setDefaultActionListener(this);
        menuBar    = menuLoader.createMenuBar(menuKey);
        if (menuBar != null) cp.add("North",menuBar);
      } // end if
      done = true;
    } finally {
      if (!done) dispose();
    }
  } // end constructor JMenuTestFrame

  /** Insert a string into the embedded editor. */
  protected void writeToEditor(String text) {
    editor.setEditable(true);
    editor.replaceSelection(text + "\n");
    editor.setEditable(false);
  } // end writeToError

  // methods
  // --- ActionListener ---------------------------------------------
  /** Invoked when an action occurs. */
  public void actionPerformed(ActionEvent e) {
    Object obj = e.getSource();
    String str = e.getActionCommand();

    if (obj instanceof JCheckBoxMenuItem) {
      if (((JCheckBoxMenuItem)obj).getState()) str += " selected";
        else str += " unselected";
    } // end if
    writeToEditor(str);
  } // end actionPerformed

} // end JMenuTestFrame

/* ******************************************************************
   end of file
******************************************************************* */
