/* ******************************************************************

   class    : ErrorMessage
   date     : 24.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Messages;

/* ******************************************************************
   import statements
*********************************************************************/

/**
 * A message that reports a summary, a warning or an error.
 *
 * @author puehringer edgar
 */
public class ErrorMessage extends Message {

  // class variable declarations
  /** Indicates that this message is a normal message. */
  public final static int MESSAGE = LOW;
  /** Indicates that this message is a warning message. */
  public final static int WARNING = NORMAL;
  /** Indicates that this message is an error message. */
  public final static int ERROR = HIGH;

  // instance variable declarations
  /** A string that contains extra info to display in a tooltip. */
  public String popupInfo;

  // constructors
  /** Create a new ErrorMessage object. */
  public ErrorMessage(int type, String msg) {
    this(type,msg,null);
  } // end constructor ErrorMessage

  /** Create a new ErrorMessage object. */
  public ErrorMessage(int type, String msg, String popupInfo) {
    super(msg,type);
    this.popupInfo = popupInfo;
  } // end constructor ErrorMessage

  // methods
  /** Return true if this is a warning. */
  public final boolean isWarning() {
    return (priority == WARNING);
  } // end isWarning

  /** Return true if this is an error, not a warning. */
  public final boolean isError() {
    return (priority == ERROR);
  } // end isError

  /** Returns the popup information. */
  public String getPopupInfo() {
    return popupInfo;
  } // end getPopupInfo

} // end ErrorMessage

/* ******************************************************************
   end of file
*********************************************************************/
