// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import SID.Utils.*;

// a node in the hierarchy tree of the classes
// the tree can handle single inheritance
public class ClassNode{

	private ClassNode superclass;
	private Vector subclasses;
	public ClassInfo thisClass;

	public boolean mark;			// temp. mark when the tree is built up

	protected ClassNode(){
		unchain();
		thisClass = null;
	}

	protected ClassNode(ClassInfo thisClass){
		unchain();
		this.thisClass = thisClass;
	}

	// if class already exists - replace old class
	void addSubClass(ClassNode cn){
		for (int i=0; i < subclasses.size(); i++){
			ClassNode n = (ClassNode)subclasses.elementAt(i);
			if (cn.thisClass.name.equals(n.thisClass.name)){
				subclasses.removeElementAt(i);
				break;
			}
		}
		subclasses.addElement(cn);
	}

	void removeSubClass(ClassInfo thisClass){
		for (int i=0; i < subclasses.size(); i++){
			ClassNode n = (ClassNode)subclasses.elementAt(i);
			if (thisClass.name.equals(n.thisClass.name)){
				subclasses.removeElementAt(i);
				return;
			}
		}
	}

	void addSuperClass(ClassNode cn){
		superclass = cn;
	}

	void unchain(){
		superclass = null;
		subclasses = new Vector(2,1);
	}

	Enumeration subClasses(){
		return subclasses.elements();
	}

	ClassInfo superClass(){
		if (superclass == null) return null;
		return superclass.thisClass;
	}
}