// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import SID.Utils.*;

// base class for method representation
// 
// no resolve of names is done, output in java style order
//
public abstract class MethodInfo extends DocumentedItem{

	protected static modifierPool modResolve;
	protected static modifierPool typeResolve;
	protected static int modMax;				// maximum number of modifiers

	//public String name;
	public short retType;
	protected BitSet modifiers;
	protected Vector parameters;

	public MethodInfo(){
		super();
		this.retType = 0;
		modifiers = new BitSet(10);
		parameters = new Vector(4,3);
	}

	public MethodInfo(String name, short retType){
		super(name);
		this.retType = retType;
		modifiers = new BitSet(10);
		parameters = new Vector(4,3);
	}

	public MethodInfo(String name, String retType){
		this.name = name;
		this.retType = (short)typeResolve.mapMod(retType);
		modifiers = new BitSet(10);
		parameters = new Vector(4,3);
	}

	public void setModifier(int mod){
		modifiers.set(mod);
	}

	public boolean isModifierSet(int mod){
		return modifiers.get(mod);
	}

	public boolean isModifierSet(String mod){
		return modifiers.get(modResolve.mapMod(mod));
	}

	public void removeModifier(int mod){
		modifiers.clear(mod);
	}

	public void setModifiers(BitSet mods){
		modifiers = new BitSet();
		modifiers.or(mods);
	}

	public void addParameter(Parameter par){
		parameters.addElement(par);
	}

	public void removeParameter(Parameter par){
		for(int i = 0; i< parameters.size(); i++){
			if (par.equals((Parameter)parameters.elementAt(i))){
				parameters.removeElementAt(i);
				return;
			}
		}
	}

	public Enumeration parameters(){ return parameters.elements(); }

	// java style output
	public String toLongString(){
		StringBuffer retString = new StringBuffer("");
		String mods = modifiers.toString();
		retString.append(modResolve.resolveModifiers(modifiers));
		retString.append(typeResolve.mapMod(retType) + " " + name + "(");
		
		for (Enumeration e=parameters.elements(); e.hasMoreElements();){
			Parameter p = (Parameter)e.nextElement();

			retString.append(p.toString());
			if (e.hasMoreElements()) retString.append(", ");
		}
		return  (retString.toString()+")");
	}

	public void load(DataInputStream in) throws IOException{
		super.load(in);
		modifiers = tool.readBitSet(in);
		name = in.readUTF();
		retType = in.readShort();

		int size = in.readShort();
		Parameter par = null;

		parameters = new Vector(size);
		if (size > 0){
			Object o = tool.readClassInfo(in);
			Class c = o.getClass();
			for (int i=0; i<size; i++){
				try{
					par = (Parameter)(c.newInstance());
				}
				catch (InstantiationException e1) { e1.printStackTrace(); }
				catch (IllegalAccessException e1) { e1.printStackTrace(); }
				par.load(in);
				parameters.addElement(par);
			}
		}
	}

	public void store(DataOutputStream out) throws IOException{
		super.store(out);
		tool.writeBitSet(modifiers, modMax, out);
		out.writeUTF(name);
		out.writeShort(retType);

		out.writeShort((short)parameters.size());
		Enumeration e = parameters.elements(); 
		if (e.hasMoreElements()){
			Parameter par = (Parameter)e.nextElement();
			tool.writeClassInfo(par, out);
//			System.out.println("writing "+ai.name);
			par.store(out);
		
			for (;e.hasMoreElements();){
				par = (Parameter)e.nextElement();
//				System.out.println("writing "+ai.name);
				par.store(out);
			}
		}
	}
}