package SID.Parser;
import java.io.*;
import java.util.*;

class Token {
	int kind;		// token kind
	int pos;		// token position in the source text (starting at 0)
	int col;		// token column (starting at 0)
	int line;		// token line (starting at 1)
	String val;		// token value
}

class Buffer {
	static byte[] buf;
	static int bufLen;
	static int pos;
	
	static void Fill(String name) throws IOException{
		InputStream s;
		int n;

		s = new FileInputStream(name);
		bufLen = s.available();
		s = new BufferedInputStream(s, bufLen);
		buf = new byte[bufLen];
		n = s.read(buf); pos = 0;
	}

	static void Fill(StringReader in) throws IOException{
		StringBuffer sbuf = new StringBuffer();
		char[] buffer = new char[4096];

		int read = in.read(buffer);
		while(read != -1){
			sbuf.append(buffer);
			read = in.read(buffer);
		}
		buf = sbuf.toString().getBytes();
		pos = 0;
		bufLen = buf.length;
	}	

	static void Fill(InputStream in) throws IOException{
		StringBuffer sbuf = new StringBuffer();
		byte[] buffer = new byte[4096];

		DataInputStream din = new DataInputStream(in);
		
		String temp = "";
		int read = din.read(buffer);
		while(read != -1){
			temp = new String(buffer,0,read);
			sbuf.append(temp);
			read = din.read(buffer);
		}
		buf = sbuf.toString().getBytes();
		pos = 0;
		bufLen = buf.length;
	}
	
	static void Set(int position) {
		if (position < 0) position = 0; else if (position >= bufLen) position = bufLen;
		pos = position;
	}
	
	static char read() {
		if (pos < bufLen){
			return (char) buf[pos++];
		}
		else
			return 65535;
	}
}

public class Scanner {

	static final char EOF = '\0';
	static final char EOL = '\n';
	static final int noSym = 85;
	static final int[] start = {
 82,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  0, 54, 24,  0,  0, 66, 50, 15, 43, 44, 37, 63, 38, 64, 11, 65,
 33, 32, 32, 32, 32, 32, 32, 32, 32, 32, 47, 36, 56, 45, 57, 46,
  0,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 41,  0, 42, 52,  1,
  0,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 39, 48, 40, 69,  0,
  0};


	public static ErrorStream err;	// error messages
	static Token t;			// current token
	static char ch;			// current input character
	static int pos;			// position of current character
	static int line;		// line number of current character
	static int lineStart;	// start position of current line
	static int oldEols;		// >0: no. of EOL in a comment;
	static BitSet ignore;	// set of characters to be ignored by the scanner

	public static void Init (String file, ErrorStream e) throws IOException{
		ignore = new BitSet(128);
		ignore.set(9); ignore.set(10); ignore.set(12); ignore.set(13); ignore.set(32); 
		err = e;
		Buffer.Fill(file);
		pos = -1; line = 1; lineStart = 0;
		oldEols = 0;
		NextCh();
	}

	public static void Init(InputStream file, ErrorStream e) throws IOException{
		ignore = new BitSet(128);
		ignore.set(9); ignore.set(10); ignore.set(12); ignore.set(13); ignore.set(32); 
		err = e;
		Buffer.Fill(file);
		pos = -1; line = 1; lineStart = 0;
		oldEols = 0;
		NextCh();
	}

	public static void Init(StringReader file, ErrorStream e) throws IOException{
		ignore = new BitSet(128);
		ignore.set(9); ignore.set(10); ignore.set(12); ignore.set(13); ignore.set(32); 
		err = e;
		Buffer.Fill(file);
		pos = -1; line = 1; lineStart = 0;
		oldEols = 0;
		NextCh();
	}
	
	public static void Init (String file) throws IOException{
		Init(file, new ErrorStream());
	}
	
	private static void NextCh() {
		if (oldEols > 0) {
			ch = EOL; oldEols--;
		} else {
			ch = Buffer.read(); pos++;
			if (ch==EOL) { line++; lineStart = pos + 1;}
		}
		if (ch > '\u007f') {
			if (ch == '\uffff') ch = EOF;
			else {
			//	System.out.println("-- invalid character at line " + line + " col " + (pos - lineStart) + " is ignored.");
				ch = ' ';
			}
		}
	}
	
	private static boolean Comment() {
		int level, line0, lineStart0; char startCh;
		level = 1; line0 = line; lineStart0 = lineStart;
		if (ch=='/') {
			NextCh();
			if (ch=='/') {
				NextCh();
				for(;;) {
					if (ch==10) {
						level--;
						if (level==0) {oldEols=line-line0; NextCh(); return true;}
						NextCh();
					} else if (ch==EOF) return false;
					else NextCh();
				}
			} else {
				if (ch==EOL) {line--; lineStart = lineStart0;}
				pos = pos - 2; Buffer.Set(pos+1); NextCh();
			}
		}		if (ch=='/') {
			NextCh();
			if (ch=='/') {
				NextCh();
				for(;;) {
					if (ch==13) {
						level--;
						if (level==0) {oldEols=line-line0; NextCh(); return true;}
						NextCh();
					} else if (ch==EOF) return false;
					else NextCh();
				}
			} else {
				if (ch==EOL) {line--; lineStart = lineStart0;}
				pos = pos - 2; Buffer.Set(pos+1); NextCh();
			}
		}		if (ch=='/') {
			NextCh();
			if (ch=='*') {
				NextCh();
				for(;;) {
					if (ch=='*') {
						NextCh();
						if (ch=='/') {
							level--;
							if (level==0) {oldEols=line-line0; NextCh(); return true;}
							NextCh();
						}
					} else if (ch==EOF) return false;
					else NextCh();
				}
			} else {
				if (ch==EOL) {line--; lineStart = lineStart0;}
				pos = pos - 2; Buffer.Set(pos+1); NextCh();
			}
		}
		return false;
	}
	
	private static void CheckLiteral() {
		switch (t.val.charAt(0)) {
			case 'a': {
				if (t.val.equals("abstract")) t.kind = 15;
				break;}
			case 'b': {
				if (t.val.equals("boolean")) t.kind = 35;
				else if (t.val.equals("byte")) t.kind = 36;
				break;}
			case 'c': {
				if (t.val.equals("char")) t.kind = 40;
				else if (t.val.equals("class")) t.kind = 21;
				break;}
			case 'd': {
				if (t.val.equals("double")) t.kind = 42;
				break;}
			case 'e': {
				if (t.val.equals("extends")) t.kind = 22;
				break;}
			case 'f': {
				if (t.val.equals("false")) t.kind = 69;
				else if (t.val.equals("final")) t.kind = 16;
				else if (t.val.equals("float")) t.kind = 41;
				break;}
			case 'i': {
				if (t.val.equals("implements")) t.kind = 23;
				else if (t.val.equals("import")) t.kind = 8;
				else if (t.val.equals("instanceof")) t.kind = 56;
				else if (t.val.equals("int")) t.kind = 38;
				else if (t.val.equals("interface")) t.kind = 27;
				break;}
			case 'l': {
				if (t.val.equals("long")) t.kind = 39;
				break;}
			case 'n': {
				if (t.val.equals("native")) t.kind = 17;
				else if (t.val.equals("new")) t.kind = 73;
				else if (t.val.equals("null")) t.kind = 70;
				break;}
			case 'p': {
				if (t.val.equals("package")) t.kind = 6;
				else if (t.val.equals("private")) t.kind = 13;
				else if (t.val.equals("protected")) t.kind = 12;
				else if (t.val.equals("public")) t.kind = 11;
				break;}
			case 's': {
				if (t.val.equals("short")) t.kind = 37;
				else if (t.val.equals("static")) t.kind = 14;
				else if (t.val.equals("super")) t.kind = 72;
				else if (t.val.equals("synchronized")) t.kind = 18;
				break;}
			case 't': {
				if (t.val.equals("this")) t.kind = 71;
				else if (t.val.equals("throws")) t.kind = 33;
				else if (t.val.equals("transient")) t.kind = 19;
				else if (t.val.equals("true")) t.kind = 68;
				break;}
			case 'v': {
				if (t.val.equals("void")) t.kind = 30;
				else if (t.val.equals("volatile")) t.kind = 20;
				break;}
		}
	}
	
	static Token Scan() {
		int state, apx, i;
		StringBuffer buf;
		while (ignore.get((int)ch))	NextCh();
		if ((ch=='/' || ch=='/' || ch=='/') && Comment()) return Scan();
		t = new Token();
		t.pos = pos; t.col = pos - lineStart; t.line = line; 
		buf = new StringBuffer();
		state = start[ch];
		apx = 0;
		loop: for (;;) {
			buf.append(ch);
			NextCh();
			switch (state) {
				case 0: {t.kind = noSym; break loop;} // NextCh already done
				case 1:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='Z' || ch=='_' || ch>='a' && ch<='z') {break;}
					else {t.kind = 1; t.val = buf.toString(); CheckLiteral(); break loop;}
				case 2:
					{t.kind = 2; break loop;}
				case 3:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 4; break;}
					else {t.kind = noSym; break loop;}
				case 4:
					if (ch=='L' || ch=='l') {state = 2; break;}
					else if(ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {break;}
					else {t.kind = 2; break loop;}
				case 5:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='E' || ch=='e') {state = 6; break;}
					else if(ch>='0' && ch<='9') {break;}
					else {t.kind = 3; break loop;}
				case 6:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='+' || ch=='-' || ch>='0' && ch<='9') {state = 7; break;}
					else {t.kind = 3; break loop;}
				case 7:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch>='0' && ch<='9') {break;}
					else {t.kind = 3; break loop;}
				case 8:
					{t.kind = 3; break loop;}
				case 9:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='+' || ch=='-' || ch>='0' && ch<='9') {state = 10; break;}
					else {t.kind = 3; break loop;}
				case 10:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch>='0' && ch<='9') {break;}
					else {t.kind = 3; break loop;}
				case 11:
					if (ch>='0' && ch<='9') {state = 12; break;}
					else {t.kind = 9; break loop;}
				case 12:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='E' || ch=='e') {state = 13; break;}
					else if(ch>='0' && ch<='9') {break;}
					else {t.kind = 3; break loop;}
				case 13:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='+' || ch=='-' || ch>='0' && ch<='9') {state = 14; break;}
					else {t.kind = 3; break loop;}
				case 14:
					if (ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch>='0' && ch<='9') {break;}
					else {t.kind = 3; break loop;}
				case 15:
					if (ch<=9 || ch>=11 && ch<=12 || ch>=14 && ch<='&' || ch>='(' && ch<='[' || ch>=']') {state = 16; break;}
					else if(ch==92) {state = 18; break;}
					else {t.kind = noSym; break loop;}
				case 16:
					if (ch==39) {state = 17; break;}
					else {t.kind = noSym; break loop;}
				case 17:
					{t.kind = 4; break loop;}
				case 18:
					if (ch>='0' && ch<='7') {state = 19; break;}
					else if(ch<=9 || ch>=11 && ch<=12 || ch>=14 && ch<='/' || ch>='8' && ch<='t' || ch>='v') {state = 16; break;}
					else if(ch=='u') {state = 20; break;}
					else {t.kind = noSym; break loop;}
				case 19:
					if (ch>='0' && ch<='7') {break;}
					else if(ch==39) {state = 17; break;}
					else {t.kind = noSym; break loop;}
				case 20:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 21; break;}
					else {t.kind = noSym; break loop;}
				case 21:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 22; break;}
					else {t.kind = noSym; break loop;}
				case 22:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 23; break;}
					else {t.kind = noSym; break loop;}
				case 23:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 16; break;}
					else {t.kind = noSym; break loop;}
				case 24:
					if (ch<=9 || ch>=11 && ch<=12 || ch>=14 && ch<='!' || ch>='#' && ch<='[' || ch>=']') {break;}
					else if(ch=='"') {state = 27; break;}
					else if(ch==92) {state = 25; break;}
					else {t.kind = noSym; break loop;}
				case 25:
					if (ch>='0' && ch<='7') {state = 26; break;}
					else if(ch<=9 || ch>=11 && ch<=12 || ch>=14 && ch<='/' || ch>='8' && ch<='t' || ch>='v') {state = 24; break;}
					else if(ch=='u') {state = 28; break;}
					else {t.kind = noSym; break loop;}
				case 26:
					if (ch<=9 || ch>=11 && ch<=12 || ch>=14 && ch<='!' || ch>='#' && ch<='/' || ch>='8' && ch<='[' || ch>=']') {state = 24; break;}
					else if(ch>='0' && ch<='7') {state = 34; break;}
					else if(ch=='"') {state = 27; break;}
					else if(ch==92) {state = 25; break;}
					else {t.kind = noSym; break loop;}
				case 27:
					{t.kind = 5; break loop;}
				case 28:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 29; break;}
					else {t.kind = noSym; break loop;}
				case 29:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 30; break;}
					else {t.kind = noSym; break loop;}
				case 30:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 31; break;}
					else {t.kind = noSym; break loop;}
				case 31:
					if (ch>='0' && ch<='9' || ch>='A' && ch<='F' || ch>='a' && ch<='f') {state = 24; break;}
					else {t.kind = noSym; break loop;}
				case 32:
					if (ch=='L' || ch=='l') {state = 2; break;}
					else if(ch>='0' && ch<='9') {break;}
					else if(ch=='E' || ch=='e') {state = 9; break;}
					else if(ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='.') {state = 5; break;}
					else {t.kind = 2; break loop;}
				case 33:
					if (ch=='L' || ch=='l') {state = 2; break;}
					else if(ch>='8' && ch<='9') {state = 32; break;}
					else if(ch>='0' && ch<='7') {state = 35; break;}
					else if(ch=='E' || ch=='e') {state = 9; break;}
					else if(ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='X' || ch=='x') {state = 3; break;}
					else if(ch=='.') {state = 5; break;}
					else {t.kind = 2; break loop;}
				case 34:
					if (ch>='0' && ch<='7') {break;}
					else if(ch<=9 || ch>=11 && ch<=12 || ch>=14 && ch<='!' || ch>='#' && ch<='/' || ch>='8' && ch<='[' || ch>=']') {state = 24; break;}
					else if(ch=='"') {state = 27; break;}
					else if(ch==92) {state = 25; break;}
					else {t.kind = noSym; break loop;}
				case 35:
					if (ch=='L' || ch=='l') {state = 2; break;}
					else if(ch>='8' && ch<='9') {state = 32; break;}
					else if(ch>='0' && ch<='7') {break;}
					else if(ch=='E' || ch=='e') {state = 9; break;}
					else if(ch=='D' || ch=='F' || ch=='d' || ch=='f') {state = 8; break;}
					else if(ch=='.') {state = 5; break;}
					else {t.kind = 2; break loop;}
				case 36:
					{t.kind = 7; break loop;}
				case 37:
					if (ch=='=') {state = 70; break;}
					else {t.kind = 10; break loop;}
				case 38:
					{t.kind = 24; break loop;}
				case 39:
					{t.kind = 25; break loop;}
				case 40:
					{t.kind = 26; break loop;}
				case 41:
					{t.kind = 28; break loop;}
				case 42:
					{t.kind = 29; break loop;}
				case 43:
					{t.kind = 31; break loop;}
				case 44:
					{t.kind = 32; break loop;}
				case 45:
					if (ch=='=') {state = 53; break;}
					else {t.kind = 34; break loop;}
				case 46:
					{t.kind = 43; break loop;}
				case 47:
					{t.kind = 44; break loop;}
				case 48:
					if (ch=='|') {state = 49; break;}
					else if(ch=='=') {state = 81; break;}
					else {t.kind = 47; break loop;}
				case 49:
					{t.kind = 45; break loop;}
				case 50:
					if (ch=='&') {state = 51; break;}
					else if(ch=='=') {state = 79; break;}
					else {t.kind = 49; break loop;}
				case 51:
					{t.kind = 46; break loop;}
				case 52:
					if (ch=='=') {state = 80; break;}
					else {t.kind = 48; break loop;}
				case 53:
					{t.kind = 50; break loop;}
				case 54:
					if (ch=='=') {state = 55; break;}
					else {t.kind = 67; break loop;}
				case 55:
					{t.kind = 51; break loop;}
				case 56:
					if (ch=='=') {state = 58; break;}
					else if(ch=='<') {state = 61; break;}
					else {t.kind = 52; break loop;}
				case 57:
					if (ch=='=') {state = 59; break;}
					else if(ch=='>') {state = 60; break;}
					else {t.kind = 53; break loop;}
				case 58:
					{t.kind = 54; break loop;}
				case 59:
					{t.kind = 55; break loop;}
				case 60:
					if (ch=='>') {state = 62; break;}
					else if(ch=='=') {state = 77; break;}
					else {t.kind = 57; break loop;}
				case 61:
					if (ch=='=') {state = 76; break;}
					else {t.kind = 58; break loop;}
				case 62:
					if (ch=='=') {state = 78; break;}
					else {t.kind = 59; break loop;}
				case 63:
					if (ch=='+') {state = 67; break;}
					else if(ch=='=') {state = 74; break;}
					else {t.kind = 60; break loop;}
				case 64:
					if (ch=='-') {state = 68; break;}
					else if(ch=='=') {state = 75; break;}
					else {t.kind = 61; break loop;}
				case 65:
					if (ch=='+') {state = 71; break;}
					else {t.kind = 62; break loop;}
				case 66:
					if (ch=='=') {state = 73; break;}
					else {t.kind = 63; break loop;}
				case 67:
					{t.kind = 64; break loop;}
				case 68:
					{t.kind = 65; break loop;}
				case 69:
					{t.kind = 66; break loop;}
				case 70:
					{t.kind = 74; break loop;}
				case 71:
					if (ch=='=') {state = 72; break;}
					else {t.kind = noSym; break loop;}
				case 72:
					{t.kind = 75; break loop;}
				case 73:
					{t.kind = 76; break loop;}
				case 74:
					{t.kind = 77; break loop;}
				case 75:
					{t.kind = 78; break loop;}
				case 76:
					{t.kind = 79; break loop;}
				case 77:
					{t.kind = 80; break loop;}
				case 78:
					{t.kind = 81; break loop;}
				case 79:
					{t.kind = 82; break loop;}
				case 80:
					{t.kind = 83; break loop;}
				case 81:
					{t.kind = 84; break loop;}
				case 82: {t.kind = 0; break loop;}
			}
		}
		t.val = buf.toString();
		return t;
	}

}
