/* ******************************************************************

   class    : SwitchableSecurityManager
   date     : 28.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import Utils.AllowAllSecurityManager;

/**
 * A security manager that locks the system environment (exit, access to
 * system properties) and can be turned on and off.
 *
 * @author puehringer edgar
 */
public class SwitchableSecurityManager extends AllowAllSecurityManager {

  // instance variable declarations
  private boolean enabled = false;

  // constructors
  /** Create a new SwitchableSecurityManager object. */
  public SwitchableSecurityManager() {
    super();
  } // end constructor SwitchableSecurityManager

  // methods
  /**
   * Turns the security manager on or off.
   */
  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  } // end setEnabled

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * cause the Java Virtual Machine to halt with the specified status code.
   */
  public void checkExit(int status) throws SecurityException {
    if (enabled) throw new SecurityException();
  } // end checkExit

  /**
   * Throws a SecurityException if the calling thread is not allowed to
   * access or modify the system properties.
   */
  public void checkPropertiesAccess() {
    throw new SecurityException();
  } // end checkPropertiesAccess

} // end SwitchableSecurityManager

/* ******************************************************************
   end of file
*********************************************************************/
