/* ******************************************************************

   class    : ToolLoader
   date     : Okt 05 1998
   compiler : javac (1.1.6)

******************************************************************* */

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.Frame;
import java.util.*;

import com.sun.java.swing.Action;
import com.sun.java.swing.text.TextAction;

import Utils.ResourceUtil;

/**
 * This class loads all tools which are instance of
 * AbstractTool like specified in a resource file.
 *
 * @author puehringer edgar
 */
public class ToolLoader extends Observable
  implements Observer {

  // class variable declarations
  /** Prefix of the tool description resource. */
  public static final String TOOLDESC_PREFIX = "externTool";

  // instance variable declarations
  /** Vector containing all tools. */
  protected final Vector vector;

  // constructors
  /** Create a new ToolLoader object. */
  public ToolLoader(ResourceBundle resources, Properties project) {
    int i      = 0;
    String str = ResourceUtil.getResourceString(resources,
                   TOOLDESC_PREFIX + (++i));

    vector = new Vector();
    while (str != null) {
      String[] aStr = ResourceUtil.splitToWords(str);
      if (aStr.length == 2) {
        AbstractTool tool;

        tool = AbstractTool.getInstance(aStr[1],resources,project,aStr[0]);
        if (tool != null) {
          vector.addElement(tool);
          tool.addObserver(this);
        } // end if
      }
      else {
        System.err.println("ToolLoader: Illegal tool description: \"" +
                           str + "\".");
      } // end if
      str = ResourceUtil.getResourceString(resources,TOOLDESC_PREFIX + (++i));
    } // end while
  } // end constructor ToolLoader

  // methods
  /** Returns all actions of all loaded tools. */
  public Action[] getActions() {
    Action[] actions = new Action[0];
    Action[] tmp;

    for (int i=0; i<vector.size(); i++) {
      tmp     = ((AbstractTool)vector.elementAt(i)).getActions();
      actions = TextAction.augmentList(actions,tmp);
    } // end for
    return actions;
  } // end getActions

  /** Sets the parent frame for all loaded tools. */
  public void setParentFrame(Frame frame) {
    for (int i=0; i<vector.size(); i++)
      ((AbstractTool)vector.elementAt(i)).setParentFrame(frame);
    // end for
  } // end setParentFrame

  // --- Observer ---------------------------------------------------
  /**
   * This method is called whenever the observed object is changed.
   * An application calls an observable object's notifyObservers
   * method to have all the object's observers notified of the change.
   */
  public void update(Observable o, Object arg) {
    setChanged();
    notifyObservers(arg);
  } // end update
} // end ToolLoader

/* ******************************************************************
   end of file
******************************************************************* */
