/* ******************************************************************

   class    : Assert
   date     : Jn 21 1999
   compiler : javac (1.1.6)

******************************************************************* */

package Utils;

/**
 * A class that acts like the C/C++ assertion macro.
 * For a detailed description of this topic look at
 * <A HREF="http://www.afu.com/javafaq.html">
 * http://www.afu.com/javafaq.html</A> and search for
 * "11.6 How can I write C/C++ style assertions in Java?".
 * All <code>assert</code> methods throw an AssertException, if
 * the asserted condition is not true and the "Enable assertions"
 * checkbox was selected while creating the class from the template.
 *
 * @see AssertionException
 * @author Puehringer Edgar
 */
public final class Assert {

  // class variable declarations
  /** True if assert methods are enabled. */
  public static final boolean ENABLED = false;

  // constructors
  // avoid creating instances of this class
  private Assert() {
  } // end constructor Assert

  // class methods
  /**
   * Assert that the boolean condition <code>b</code> is true.
   *
   * @param b The boolean condition to check.
   */
  public static final void assert(boolean b) {
    // Assertions disabled - do nothing
  } // end assert

  /**
   * Assert that the boolean condition <code>b</code> is true.
   *
   * @param b The boolean condition to check.
   * @param s The detail message for the resulting exception.
   */
  public static final void assert(boolean b, String s) {
    // Assertions disabled - do nothing
  } // end assert

  /**
   * Assert that the long parameter <code>l</code> is not zero.
   *
   * @param l The long value to check.
   */
  public static final void assert(long l) {
    // Assertions disabled - do nothing
  } // end assert

  /**
   * Assert that the long parameter <code>l</code> is not zero.
   *
   * @param l The long value to check.
   * @param s The detail message for the resulting exception.
   */
  public static final void assert(long l, String s) {
    // Assertions disabled - do nothing
  } // end assert

  /**
   * Assert that the object reference <code>l</code> is not
   * <code>null</code>.
   *
   * @param ref The object reference to check.
   */
  public static final void assert(Object ref) {
    // Assertions disabled - do nothing
  } // end assert

  /**
   * Assert that the object reference <code>l</code> is not
   * <code>null</code>.
   *
   * @param ref The object reference to check.
   * @param s The detail message for the resulting exception.
   */
  public static final void assert(Object ref, String s) {
    // Assertions disabled - do nothing
  } // end assert

} // end Assert

/* ******************************************************************
   class declarations
*********************************************************************/

/**
 * Thrown when a condition asserted from an <code>assert</code>
 * method of the <code>Assert</code> is not true.
 *
 * @see Assert
 */
class AssertionException extends Error {

  // constructors
  /** Constructs a <code>AssertionException</code> with no detail message. */
  public AssertionException() {
    super();
  } // end AssertionException

  /**
   * Constructs a <code>ClassFormatError</code> with the specified detail message.
   *
   * @param s The detail message.
   */
  public AssertionException(String s) {
    super(s);
  } // end AssertionException

} // end AssertionException

/* ******************************************************************
   end of file
*********************************************************************/
