/* ******************************************************************

   class    : FrameUtil
   date     : 13.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Utils;

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;

/**
 *
 * Helpful utility methods for handling frames.
 *
 * @author puehringer edgar
 *
 */
public class FrameUtil {

  // class variable declarations
  private final static int X_TILES  = 10;
  private final static int Y_TILES  = 10;
  private final static FrameUtil FU = new FrameUtil();

  // instance variable declarations
  private int x = 0;
  private int y = 0;

 // constructors
  private FrameUtil() {}

  // class methods
  /**
   * Returns a child of the specified class in a contain or null
   * if it doesn't exits.
   *
   * @param c The container.
   * @param clazz The class name of the child to look for.
   */
  public final static Object contains(Container c, String clazz) {
    Class cl;
    try {
      cl = Class.forName(clazz);
      return contains(c,cl);
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
    }
    return null;
  } // end contains

  /**
   * Returns a child of the specified class in a contain or null
   * if it doesn't exits.
   *
   * @param c The container.
   * @param clazz The class of the child to look for.
   */
  public final static Object contains(Container c, Class clazz) {
    Component[] comp = c.getComponents();
    Object obj;

    for (int i=0; i<comp.length; i++) {
      if (clazz.isInstance(comp[i])) {
        return comp[i];
      } else if (comp[i] instanceof Container) {
        obj = contains((Container)comp[i],clazz);
        if (obj != null) return obj;
      } // end if
    } // end for
    return null;
  } // end contains

  /**
   * Center the window wnd in the Frame frame.  If frame is null, then
   * center the window within the screen.
   *
   * @param wnd The window to center.
   * @param frame The parent frame.
   */
  public final static void center(Window wnd, Frame frame) {
    int x,y;
    Point p;
    Dimension f,d;

    if (frame == null) {
      p = new Point(0,0);
      f = wnd.getToolkit().getScreenSize();
    }
    else {
      p = frame.getLocation();
      f = frame.getSize();
    } // end if
    d = wnd.getSize();
    x = p.x + (f.width  - d.width)  / 2;
    y = p.y + (f.height - d.height) / 2;
    if (x < 0) x = 0;
    if (y < 0) y = 0;
    wnd.setLocation(x,y);
  } // end center

  /** Shuffles the window location like implemented in windows by default. */
  public final static void shuffleLocation(Window wnd) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension d      = wnd.getSize();

    if ((screen.width  < FU.x + d.width) ||
        (screen.height < FU.y + d.height)) {
      FU.x = 0;
      FU.y = 0;
    } // end if
    wnd.setLocation(FU.x,FU.y);
    FU.x += (screen.width  / X_TILES);
    FU.y += (screen.height / Y_TILES);
  } // end shuffleLocation

  /** Return the hosting frame or null if it doesn't exist. */
  public final static Frame getParentFrame(Component comp) {
    if (comp instanceof Frame) return (Frame)comp;
    for (Container p = comp.getParent(); p != null; p = p.getParent())
      if (p instanceof Frame) return (Frame) p;
    // end for
    return null;
  } // end getParentFrame

} // end FrameUtil

/* ******************************************************************
   end of file
*********************************************************************/
