/* ******************************************************************

   class    : ZipUtil
   date     : Okt 07 1998
   compiler : javac (1.1.6)

******************************************************************* */

package Utils;

/* ******************************************************************
   import statements
******************************************************************* */

import java.io.*;
import java.util.*;
import java.util.zip.*;

/**
 * This class copies the content of a directory to a zip file.
 *
 * @author puehringer edgar
 */
public class ZipUtil extends Observable {

  // class variable declarations

  // instance variable declarations
  public int nrOfFiles   = 0;
  public File curFile    = null;
  public String errorMsg = null;

  // constructors
  /** Create a new ZipUtil object. */
  public ZipUtil() {
  } // end constructor ZipUtil

  // methods
  /**
   * Adds the content of a directory to a zip file.
   *
   * @param os The output stream to write the zip file.
   * @param dir The directory to add to the zip file.
   * @param filter Only files accepted by filter are added.
   * @param recurse True if surdirectories should be included.
   * @param compress If false, files are only stored.
   */
  public void dirToZip(OutputStream os, File dir, FilenameFilter filter,
                       boolean recurse,boolean compress) {
    Vector vector;
    ZipOutputStream zos;

    if (!dir.isDirectory()) return;
    vector = new Vector();
    zos    = new ZipOutputStream(os);
    addFiles(dir,vector,filter,recurse);
    nrOfFiles = vector.size();
    try {
      for (int i=0; i<vector.size(); i++) {
        Checksum checksum;
        FileInputStream fis        = null;
        CheckedInputStream cis     = null;
        ByteArrayOutputStream baos = null;
        ZipEntry desc;
        boolean done = false;
        byte[] aByte = null;
        String str;

        curFile = (File)vector.elementAt(i);
        notifyObs(curFile);
        str = StringUtil.relativePath(curFile.getAbsolutePath(),
                                        dir.getAbsolutePath());
        desc = new ZipEntry(str);
        desc.setMethod(compress ? ZipEntry.DEFLATED : ZipEntry.STORED);
        desc.setSize(curFile.length());
        desc.setTime((new Date()).getTime());
        try {
          fis      = new FileInputStream(curFile);
          checksum = new CRC32();
          cis      = new CheckedInputStream(fis,checksum);
          baos     = new ByteArrayOutputStream();
          copy(cis,baos);
          aByte = baos.toByteArray();
          desc.setCrc(checksum.getValue());
          done = true;
        } catch (IOException excep) {
          errorMsg = "Exception on reading \"" + curFile.getName() +
                     "\": " + excep;
          notifyObs(errorMsg);
        } finally {
          if (baos != null) try { baos.close(); } catch (IOException e){}
          if (cis != null)  try { cis.close();  } catch (IOException e){}
          if (fis != null)  try { fis.close();  } catch (IOException e){}
        }
        if (done) {
          ByteArrayInputStream bais = new ByteArrayInputStream(aByte);

          zos.putNextEntry(desc);
          copy(bais,zos);
          zos.closeEntry();
        } // end if
      } // end for
    } catch (IOException excep) {
          errorMsg = "Exception on writing jar file: " + excep;
          notifyObs(errorMsg);
    } finally {
      try { zos.close(); } catch (IOException e) {}
    }
    if (nrOfFiles > 0) notifyObs(null);
  } // end dirToZip

  private final void notifyObs(Object obj) {
    setChanged();
    notifyObservers(obj);
  } // end notifyObs

  private void addFiles(File dir, Vector vector,
                        FilenameFilter filter, boolean recurse) {
    String[] names = dir.list();

    for (int i=0; i<names.length; i++) {
      File file = new File(dir,names[i]);

      if (file.isDirectory()) {
        if (recurse)
          addFiles(file,vector,filter,recurse);
        // end if
      }
      else {
        if (filter.accept(dir,names[i]))
          vector.addElement(file);
        // end if
      } // end if
    } // end for
  } // end addFiles

  private final void copy(InputStream is, OutputStream os)
    throws IOException {
    byte[] buff  = new byte[4096];
    int nch;

    while ((nch = is.read(buff,0,buff.length)) != -1) {
      os.write(buff,0,nch);
    } // end while
  } // end copy

} // end ZipUtil

/* ******************************************************************
   end of file
******************************************************************* */
